;Very small composite video signal generator
;5 lines, 10 chars each, static text (set text at the end of this file)
;Charset: 5x7 pixels (+1 pixel spacing in both directions), ASCII 0x20-0x7F
;ATTiny11(L) overclocked to 8MHz (external XTAL), Fuse Byte = 0x1F
;          ___
;  PB0 ---|___|---o-----> Composite Output (load with 75 ohms)
;          1k     |
;          ___    |
;  PB1 ---|___|---'
;          390
;
;Updated 2008-03-28 by Arne Rossius
;  * BUGFIX: removed additional delay after VSYNC pulses
;  * BUGFIX: calculation of LastLine always assumed Lines == 5
;  * stabilised image some more

.include "../include/tn11def.inc"

;R0 needed for LPM
;R1..R10 needed for "RAM" shadow of one line (10 chars)
.def	zero = R11

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	lineL = R19
.def	lineH = R20
.def	charline = R21
.def	linediv = R22
.def	charsetL = R23
.def	charsetH = R24
.def	textpos = R25
.def	ramptr = R26
.def	sync_out = R27

;R31:R30 needed for pointer (Z)

.equ	Lines = 5
.equ	LineDivInit = 576/(2*Lines*8)-1
.equ	FirstLine = 18+(288-Lines*LineDivInit*8)/2
.equ	LastLine = FirstLine + Lines*8*LineDivInit

.equ	SYNC = 0 ;PORTB0
.equ	DATA = 1 ;PORTB1

;===============================================================================

.cseg
.org 0x000
	clr	zero
	ldi	sync_out, 1<<SYNC
	rjmp	reset
	
.org OVF0addr ;.org 0x003
	;check if vsync is needed
	ldi	temp, 1
	add	lineL, temp
	adc	lineH, zero
	ldi	temp, HIGH(311)
	cpi	lineL, LOW(311)
	cpc	lineH, temp
	brlo	not_vsync

	;reload timer for VSYNC (less than 64 cycles after interrupt call)
	ldi	temp, 256-8*8 ;VSYNC pulse is the length of 8 lines
	out	TCNT0, temp
	
	;vertical synchronisation
	rcall	vsync_prepost ;5 pre pulses (short sync)
	rcall	vsync_prepost
	rcall	vsync_prepost
	rcall	vsync_prepost
	rcall	vsync_prepost
	rcall	vsync_main ;5 main pulses (short black)
	rcall	vsync_main
	rcall	vsync_main
	rcall	vsync_main
	rcall	vsync_main
	rcall	vsync_prepost ;5 post pulses (short sync)
	rcall	vsync_prepost
	rcall	vsync_prepost
	rcall	vsync_prepost
	rcall	vsync_prepost
	;initialize registers
	ldi	lineL, 0
	ldi	lineH, 0
	ldi	charline, 7
	ldi	linediv, LineDivInit
	ldi	textpos, LOW(text*2)
;	;clear interrupt flag
;;	ldi	temp, 1<<TOV0
;;	out	TIFR, temp
;	out	TIFR, charline
	nop ;added for stable image
	rjmp	line_end

;--------------------
	
not_vsync:
	;reload timer for HSYNC (less than 64 cycles after interrupt call)
	ldi	temp, 256-8
	out	TCNT0, temp
	
	ldi	ramptr, 9 ;second char: R9 (first char is loaded seperately)
;	nop
;	
;	;HSYNC pulse: 4.75us = 38 cycles
;	out	PORTB, zero
	cbi	PORTB, SYNC
	ldi	temp, 12			;3*12 = 36
line_hsync:	
	dec	temp
	brne	line_hsync
;	nop					;37
;	out	PORTB, sync_out			;38
	sbi	PORTB, SYNC			;38
	
	;PRE black + 3.5us: 5.75us + 3.5us = 74 cycles
	ldi	charsetL, LOW(charset*2+96*6-0x20);1
	ldi	charsetH, HIGH(charset*2+96*6-0x20);2
	mov	temp, charline			;3
	clr	temp2				;4
	ldi	count, 5 ;multiply by 32	;4 + 5*5 = 29
line_multiply:
	lsl	temp
	rol	temp2
	dec	count
	brne	line_multiply
	sub	charsetL, temp			;30
	sbc	charsetH, temp2			;31
	lsl	temp ;multiply by 2 (-> *64)	;32
	rol	temp2				;33
	sub	charsetL, temp			;34
	sbc	charsetH, temp2			;35
	ldi	temp, HIGH(FirstLine)		;36
	cpi	lineL, LOW(FirstLine)		;37
	cpc	lineH, temp			;38
	brlo	line_end			;39
	ldi	temp, HIGH(LastLine)		;40
	cpi	lineL, LOW(LastLine)		;41
	cpc	lineH, temp			;42
	brsh	line_end			;43
	cpi	charline, 7 ;first line: empty	;44
	mov	ZL, charsetL			;46 ; > swapped mov and brsh for stable image
	brsh	line_charline			;45 ;/
	mov	ZH, charsetH			;47
	add	ZL, R10 ;start with R10		;48
	adc	ZH, zero			;49
	lpm					;52
	ldi	temp, 6				;52 + 6*3 = 70
line_wait:
	dec	temp
	brne	line_wait
	
line_loop:
	;CHAR: 6 pixels, 6 cycles each (4.5us)
	ldi	temp2, 1<<SYNC ;pixel 1		;71
	sbrc	R0, 4				;72
	ldi	temp2, 1<<SYNC | 1<<DATA	;73
	out	PORTB, temp2			;74 (first pixel starts here)
	mov	ZL, ramptr
	nop
	ldi	temp2, 1<<SYNC ;pixel 2
	sbrc	R0, 3
	ldi	temp2, 1<<SYNC | 1<<DATA
	out	PORTB, temp2
	ld	temp, Z
	ldi	temp2, 1<<SYNC ;pixel 3
	sbrc	R0, 2
	ldi	temp2, 1<<SYNC | 1<<DATA
	out	PORTB, temp2
	mov	ZL, charsetL
	mov	ZH, charsetH
	ldi	temp2, 1<<SYNC ;pixel 4
	sbrc	R0, 1
	ldi	temp2, 1<<SYNC | 1<<DATA
	out	PORTB, temp2
	add	ZL, temp
	adc	ZH, zero
	ldi	temp2, 1<<SYNC ;pixel 5
	sbrc	R0, 0
	ldi	temp2, 1<<SYNC | 1<<DATA
	out	PORTB, temp2
	lpm ;2 cycles for pixel 5, 1 cycle for pause
	subi	ramptr, 1
	ldi	temp2, 1<<SYNC
	out	PORTB, temp2
	brcc	line_loop ;stop if RAM pointer is at -1

line_charline:
	;determine next charline
	dec	linediv
	brne	line_end
	ldi	linediv, LineDivInit
	dec	charline
	andi	charline, 0x07
	;copy new text to RAM (R1-R10) after first scanline is complete
	cpi	charline, 6
	ldi	ZH, HIGH(text*2) ; > swapped ldi and brne for stable image
	brne	line_end         ;/
	ldi	ramptr, 10
line_copy_loop:
	mov	ZL, textpos
	lpm
	inc	textpos
	mov	ZL, ramptr
	st	Z, R0
	dec	ramptr
	brne	line_copy_loop
	nop ;added for stable image
	
	
line_end:
	reti
	
;-------------------------------------------------------------------------------

vsync_prepost:
	out	PORTB, zero
	;SYNC: 2.375us = 19 cycles
	ldi	temp, 6			;6*3 = 18
vsync_prepost_sync:
	dec	temp
	brne	vsync_prepost_sync
	out	PORTB, sync_out		;19
	;BLACK: 29.625us = 237 cycles
	ldi	temp, 76		;76*3 = 228
vsync_prepost_black:
	dec	temp
	brne	vsync_prepost_black
	nop				;229
	ret				;237 (incl. next 'rcall' and 'out')

;--------------------

vsync_main:
	out	PORTB, zero
	;SYNC: 27.25us = 218 cycles
	ldi	temp, 72		;72*3 = 216
vsync_sync:
	dec	temp
	brne	vsync_sync
;	nop				;217
;	out	PORTB, sync_out		;218
	sbi	PORTB, SYNC		;218
	;BLACK: 4.75us = 38 cycles
	ldi	temp, 10		;10*3 = 30
vsync_black:
	dec	temp
	brne	vsync_black
	ret				;38 (incl. next 'rcall' and 'out')
	
;===============================================================================

reset:
	;init registers
;	ldi	lineL, 0
;	ldi	lineH, 0
;	ldi	charline, 7
;	ldi	linediv, LineDivInit
;	ldi	textpos, LOW(text*2)
	
	;init PORTB
	ldi	temp, 0x1F
	out	DDRB, temp
	;out	PORTB, zero
	
	;init timer
;	ldi	temp, 256-8 ;Clk/64/8 = 15625 Hz (horizontal line frequency)
;	out	TCNT0, temp
	ldi	temp, 0x03 ;Clk/64
	out	TCCR0, temp
;	ldi	temp, 1<<TOIE0
	out	TIMSK, temp
	
	;enable interrupts
	sei
	
loop:
	rjmp	loop
	
;===============================================================================
	
charset:
	.include "charset_5x7.asm"
charset_end:

.org 0x200-(Lines*5) ;5 words per line = 10 bytes/chars per line
text:
;	.db "          "
;	.db "  CAMERA  "
;	.db "          "
;	.db "  IN USE  "
;	.db "          "

	.db "ATTiny11L "
	.db "Composite "
	.db "Video Sig."
	.db "Generator "
	.db "10x5 Chars"
